/*
  gsl_integration.cc

  That's a C++ wrapper for gsl_integration.c

  Author: Tobias Naehring
  
  g++ -DMAIN_GSL_INTEGRATION gsl_integration.cc -lgsl -lgslcblas

 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#include"gsl_integration.hh"
#include<iostream>

// Constants:
int gsl_integration_t::gsl_integration_workspace_size = 1000;
double gsl_integration_t::gsl_epsabs_default = 1.0e-6;
double gsl_integration_t::gsl_epsrel_default = 1.0e-6;
size_t gsl_integration_t::gsl_limit_default = 100;
int gsl_integration_t::gsl_key_default = GSL_INTEG_GAUSS41;

// Constructor:
gsl_integration_t::gsl_integration_t()
{
  c_gsl_integrand_struct.function = c_gsl_integrand;
  c_gsl_integrand_struct.params = (void *) this;
  // set defaults:
  gsl_epsabs = gsl_epsabs_default;
  gsl_epsrel = gsl_epsrel_default;
  gsl_key = gsl_key_default;
  gsl_limit = gsl_limit_default;
  
  /*
    Stuff for the gsl integrator itself:
  */
  c_gsl_integration_workspace =
    gsl_integration_workspace_alloc(gsl_integration_workspace_size);
}

gsl_integration_t::~gsl_integration_t()
{
  gsl_integration_workspace_free (c_gsl_integration_workspace);
}

double gsl_integration_t::c_gsl_integrand(double x, void* ptr)
{
  return (((gsl_integration_t*) ptr) -> integrand)(x);
}

double gsl_integration_t::integrate(double a, double b)
{
  double result;
  double abserr;
  int status;
  
  status = gsl_integration_qag( &c_gsl_integrand_struct, a, b, gsl_epsabs,
				gsl_epsrel,
				gsl_limit, gsl_key,
				c_gsl_integration_workspace,
				  &result, &abserr);
  if( status != 0 )
    {
	std::cerr << "\n Error in gsl_integration_qag. Error code: " 
	     << status << std::endl;
    }
  
  return result;
}

#ifdef MAIN_GSL_INTEGRATION

/*
  Testing:
*/

class F : public gsl_integration_t
{
public:
  double integrand(double x)
  {
    return x*x;
  }
};

void main()
{
  double a,b,y;
  F f;
  
  std::cout << "\n Limits: \n a="; std::cin >> a;
  std::cout << "\n b="; std::cin >> b;
  
  y= f.integrate(a,b);
  std::cout << " Result: y=" << y << std::endl;
}


#endif
