/*
  Do not modify ~/include/gsl_integration.hh
  since this file will be overwritten by
  ~/C++/Maths/gsl_integration.hh

  gsl_integration.hh

  That's a C++ wrapper for gsl_integration.h

  Author: Tobias Naehring

  Description:
  See gsl_integration.cc for an example.
  
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
#ifndef _GSL_INTEGRATION_HH__
#define _GSL_INTEGRATION_HH__
#include<gsl/gsl_integration.h>

class gsl_integration_t
{
  /*
    Hidden stuff for the gsl integrator itself:
  */
  static int gsl_integration_workspace_size; // = 1000;
  gsl_integration_workspace* c_gsl_integration_workspace;
  

  // The C-compatible function:
  gsl_function c_gsl_integrand_struct;

  // The c function representing the integrand:
  static double c_gsl_integrand(double x, void* ptr);
  
public:
  // Default gsl constants:
  static double gsl_epsabs_default; // = 1.0e-6;
  static double gsl_epsrel_default; // = 1.0e-6;
  static size_t gsl_limit_default; // = 100;
  static int gsl_key_default; // = GSL_INTEG_GAUSS41;
  
  double gsl_epsabs, gsl_epsrel;
  size_t gsl_limit;
  int gsl_key;

  // Constructor:
  gsl_integration_t();
  
  // Destructor:
  ~gsl_integration_t();

  // The function to integrate:
  virtual double integrand(double x)=0;

  // determined integral within the limits a and b:
  double integrate(double a, double b);

};

#endif // ndef _GSL_INTEGRATION_HH__


