// Do not edit the file ~/include/tobiMat.hh !!!
// Edit the file ~/C++/Math/tobiMat.hh instead!!!


#ifndef TOBIMAT_CC
#define TOBIMAT_CC
#include<iostream>
#include<valarray>

typedef std::valarray<double> t_v;

class t_m :public t_v
{
public:
  size_t m,n,N;

  t_m(size_t _m,size_t _n) : n(_n), m(_m), N(_n*_m), t_v(_n*_m)
  {}

  t_m(const t_v& v) : m(v.size()), n(1), N(v.size()), t_v(v)
  {}

  operator double ()
  {
    return operator[](0); // Assuming 1x1-matrix.
  }

  double& operator () (size_t i, size_t j)
  {
    return operator[](i+n*j);
  }

  // matrix - matrix - product:
  t_m operator * (const t_m& b) const;

  t_m& operator= (double val)
  {
    t_v::operator=(val);
    return *this;
  }

  friend std::ostream& operator << (std::ostream& os, const t_v a);
};

std::ostream& operator << (std::ostream& os, const t_v a);

// specialized matrix type (3x3)
class t_m3 : public t_m
{
public:
  t_m3(double a00, double a01, double a02,
      double a10, double a11, double a12,
       double a20, double a21, double a22) :
    t_m(3,3)
  {
    operator [] (0) = a00;
    operator [] (1) = a10;
    operator [] (2) = a20;
    operator [] (3) = a01;
    operator [] (4) = a11;
    operator [] (5) = a21;
    operator [] (6) = a02;
    operator [] (7) = a12;
    operator [] (8) = a22;
  }

  t_m3() : t_m(3,3) {}
  t_m3(const t_m& m) : t_m(m) {}
};

// 3dim. vectors:
struct t_v3 : public t_v
{
  t_v3(const t_v& v) : t_v(v) {}
  t_v3& operator = (const t_v& v)  
  { 
    operator [] (0) = v[0];
    operator [] (1) = v[1];
    operator [] (2) = v[2];    
    return *this;
  }
  t_v3() : t_v(3) {}
  t_v3(double x,double y,double z) : t_v(3)
  {
    operator [] (0) = x;
    operator [] (1) = y;
    operator [] (2) = z;
  }
  void operator () (double x,double y,double z)
  {
    operator [] (0) = x;
    operator [] (1) = y;
    operator [] (2) = z;
  }

  t_v3& operator = (double val)
  {
    t_v::operator = (val);
    return *this;
  }

  t_v3& cross(const t_v3& a, const t_v3& b)
  {
    operator [] (0) = a[1]*b[2]-a[2]*b[1];
    operator [] (1) = a[2]*b[0]-a[0]*b[2];
    operator [] (2) = a[0]*b[1]-a[1]*b[0];
    return *this;
  }
};

t_v3 inline cross(const t_v3& a, const t_v3& b)
{
  return t_v3(
	      a[1]*b[2]-a[2]*b[1],
	      a[2]*b[0]-a[0]*b[2],
	      a[0]*b[1]-a[1]*b[0]
	      );
}

double inline dot(const t_v3& a, const t_v3& b)
{
  return a[0]*b[0]+a[1]*b[1]+a[2]*b[2];
}

double norm(const t_v& v); // Euclidian norm of v

#endif // def TOBIMAT_CC
