/*
  Do not edit the file ~/include/tobiRungeKutta.hh !!!
  Edit the file ~/C++/Math/tobiRungeKutta.hh instead.
*/

#ifndef TOBIRUNGEKUTTA_HH
#define TOBIRUNGEKUTTA_HH
#include"tobiMat.hh"
#include<iostream>
#include<fstream>
/*
  Explicit embedded Runge Kutta methods for solving ode's
  d/dt x(t) = f( t, x(t) );
  (with adaptive step size control)
*/

class t_runge_kutta
{
public:
  class t_tab
  {
  public:
    int order;
    double *p_t;
    double *p_f;
    double *p_dx_1;
    double *p_dx_2;
    t_tab(int _order=0,
	double* _p_t=NULL,
	double* _p_f=NULL,
	double* _p_dx_1=NULL,
	double* _p_dx_2=NULL
	){
      order=_order;
      p_t=_p_t;
      p_f=_p_f;
      p_dx_1=_p_dx_1;
      p_dx_2=_p_dx_2;
    }
  };


  static t_tab fehlberg;
  static t_tab sarafyan;

private:
  t_v* pf;
  t_tab tab;
public:

  double stepsize; // step size
  double step_min, step_max;
  double err_max;
  double err_fac;
  double err_min_fac;
  int n; // system dimension
  double t_max;

  double t;
  t_v x;

  t_runge_kutta(
		int _n,
		double _t=0.0,
		double _t_max=1.0,
		double _step=0.0,
		double _step_max=0.0,
		double _step_min=1.0e-12,
		double _err_max=1.0e-3,
		double _err_min_fac=0.1,
		double _err_fac=0.5,
		const t_tab& _tab=fehlberg
		) : x(_n)
  {
    t=_t;
    t_max = _t_max;
    if( _step == 0.0 ) _step = t_max / 10.0;
    if( _step_max == 0.0 ) _step_max = _step;
    n=_n;
    stepsize=_step;
    step_max=_step_max;
    step_min=_step_min;
    err_max=_err_max;
    err_min_fac=_err_min_fac;
    err_fac=_err_fac;
    pf = NULL;
    set(fehlberg);
  }

  ~t_runge_kutta()
  {
    delete [] pf;
  }

  void set(const t_tab& _tab)
  {
    tab = _tab;
    if(pf!=NULL) delete [] pf;
    pf = new t_v [tab.order+1] (n);
  }

  virtual t_v f(double t, const t_v& x)=0;

  bool step(double _t_max);
  bool step(){return step(t_max);};
};

#endif // ndef TOBIRUNGEKUTTA_HH
