/*
  Autor: Tobias N"ahring
  Lizenz: GPL (siehe: http://www.gnu.org)

  Kompelieren mit:

  g++ -g plattenC-el.cc -ltobiMath

  Berechnung der Flugbahn eines Elektrons im Feld eines Plattenkondensators
  mit endlicher Ausdehnung.

  Zeitskalierung auf Nanosekunden.
 */
#include<iostream>
#include<fstream>
#include"Maths/tobiRungeKutta.hh"
#include"plattenC.cc"

const double QE_M=-1.75881962e11; 
// QE=1.60217733e-19;
const double V0=18.755e-3; // Anfangsgeschwindigkeit in m pro ns


t_plattenC plattenC;
// Geaendert am 9. 8. '04:
// (
// 		    25,     // Anzahl der Elemente in x-Richtung
// 		    25,     // Anzahl der Elemente in y-Richtung
// 		    1.5e-2, // Halbe Plattenbreite in m
// 		    0.5e-2, // Halber Plattenabstand in m
// 		    50.0    // Halbe Spannung Platte-Platte in V
// 		    );
// Die Parameter des Plattenkondensators werden nun in der Parameterdatei
// festgelegt.

class t_elektron : public t_runge_kutta
{
public:

  t_v E;

  t_elektron(): t_runge_kutta(
			      4, // Systemdimension: 2 Koord. Ort,
			      // 2 Koord. Geschw.
			      0.0, // Startzeit in ns
			      14.0, // Endzeit in ns
			      0.1 // Zeitschritt in ns
			      ),
    E(2)
  {
    /*
      Anfangsbedingungen:
      Ort:
    */
    x[0] = -3.0e-2; x[1]= 0.0;
    // Geschwindigkeit in m/ns
    // x[2]:= d/dt x[0] - V0.
    x[2] = V0; x[3]= 0.0;
    // x[2] = 1.8755e-3; x[3]= 0.0;
    set(sarafyan);
  }

  // zu integrierende Funktion:
  t_v f(double /*Zeit*/, const t_v& x)
  {
    t_v dxdt(4);

    // Zeitabl. des Ortsvektors ist Geschwindigkeit (in m/ns)
    dxdt[0] = x[2]; // dxdt[0] = x[2] + V0;
    dxdt[1] = x[3];
    
    E= plattenC.E(t_v3(x[0],0.0,x[1]))[std::slice(0,2,2)]; // x- und z- Komp.
    // cout << E[0] <<' '<< E[1] << endl;
    // Zeitabl. des Geschwindigkeitsvektors ist QE/M*E in m/(ns)^2
    dxdt[std::slice(2,2,1)]= 1.e-18 * QE_M * E ;

    return dxdt;
  }
} el;

// std::ostream& operator << (std::ostream& os, const t_v& v)
// {
//   int i;
//   if( v.size() == 0 ) return os;
//   os << v[0];
//   for(i=1;i<v.size();i++) os <<' '<< v[i];
//   return os;
// }


int main(int nargs, char* args[])
{
  int i;

  if( nargs != 2 ) {
    std::cerr << "Syntax: " << args[0] << " Parameterdatei\n"
	      << "Die Parameterdatei beinhaltet Zeilen mit Paaren von Schluesselworten "
	      << "und zugehoerigen Werten. Schluesselworte sind:\n"
	      << "n (Anzahl der Segmente in x-Richtung)\n"
	      << "m (Anzahl der Segmente in y-Richtung)\n"
	      << "a (halbe Plattenbreite)\n"
	      << "c (halber Plattenabstand)\n"
	      << "U0 (halbe Platten-Platten-Spannung)\n"
	      << "output (Name der Ausgabedatei)"
	      << std::endl;
    abort();
  }


  std::ifstream is(args[1]);
  if( !is ) {
    std::cerr << "Parameterdatei "<< args[1] << " konnte nicht geoeffnet werden."
	      << std::endl;
    abort();
  }

  // Parameter fuer Plattenkondensator festlegen:
  is >> plattenC;

  std::string os_name;
  is >> "output" >> os_name;
  std::ofstream os(os_name.c_str());

  is.close();

  while( el.t < el.t_max )
    {
      if(!el.step())
	{
	  std::cerr << "\n Integrationsschritt fehlgeschlagen!"<< std::endl;
	  abort();
	}
      os << el.t << ' '
	 << el.x << ' '
	 << el.E << ' '
	 << norm(el.x[std::slice(2,2,1)]) << ' '
	 << el.stepsize << std::endl;
    }

  os.close();
}
