/*
  Kompilation mit:
  g++ -g plattenC2.cc -ltobiMath -oplattenC
 */
#define MAIN main
#define verbose false
#include<fstream>
#include<stdio.h>
#include<math.h>
#include"Maths/tobiMath.hh"
#include<iostream>
#include<string>

/*
   tobiMat.cc:
   Einfache Matrizenrechnung (realisiert mittels valarray):
   Spaltenmatrizen: t_v
   3x1-Spaltenmatrizen: t_v3
   Matrizen: t_m
   3x3-Matrizen: t_m3
*/
#include<tobiMat.hh>

/*
  Modell des Plattenkondensators:
*/
struct t_plattenC {
  int n,m,N;
  double da,a,c,c2;
  double U0;
  double SIIconst;
  double **S;
  double *C;

  t_plattenC()
  {
    n=m=N=0;
    S=NULL;
    C=NULL;
  }

  t_plattenC(int _n, // Anzahl der Unterteilungen in x-Richtung
	     int _m, // Anzahl der Unterteilungen in y-Richtung
	     double _a, // halbe Abmessung in x-Richtung
	     double _c, // halber Plattenabstand
	     double _U0) // Potential einer Platte,
  { init(_n,_m,_a,_c,_U0); }
  /* einige Hilfsfunktionen: */

  /*
    Zusammenhang zwischen Ladung und Potential.
    (Greensche Fkt. des Laplace-Operators)
   */
  double green(double x, double y) // Version mit z=0.0
  {
    return 1.0/sqrt(x*x+y*y);
  }
  double green(double x, double y, double z)
  {
    return 1.0/sqrt(x*x+y*y+z*z);
  }

  double green(t_v& v)
  {
    return 1.0/norm(v);
  }

  /*
    Potential in einem Raumpunkt (x,y,z),
    der nicht zu nahe an den Platten liegen darf.
  */
  double pot(double x, double y, double z)
  {
    double phi=0.0;
    double xi,yi;
    int i,j;

    for(i=0;i<n;i++){
      xi=da*(0.5+i);
      for(j=0;j<m;j++){
	yi=da*(0.5+j);
	phi += C[j*n+i]*(
			 green(x-xi,y-yi,z-c)-green(x-xi,y-yi,z+c)
			 +green(x+xi,y-yi,z-c)-green(x+xi,y-yi,z+c)
			 +green(x-xi,y+yi,z-c)-green(x-xi,y+yi,z+c)
			 +green(x+xi,y+yi,z-c)-green(x+xi,y+yi,z+c)
			 );
      }
    }
  } // Ende von t_plattenC::pot(...)

  // Feldstaerke berechnen:
  t_v3 r0_case(const t_v3& r, double x0, double y0, double z0, int k)
  {
    switch(k){
    case 0: return t_v3(r[0]-x0,r[1]-y0,r[2]-z0);
    case 1: return t_v3(r[0]+x0,r[1]-y0,r[2]-z0);
    case 2: return t_v3(r[0]-x0,r[1]+y0,r[2]-z0);
    case 3: return t_v3(r[0]+x0,r[1]+y0,r[2]-z0);
    case -1: return t_v3(r[0]-x0,r[1]-y0,r[2]+z0);
    case -2: return t_v3(r[0]+x0,r[1]-y0,r[2]+z0);
    case -3: return t_v3(r[0]-x0,r[1]+y0,r[2]+z0);
    case -4: return t_v3(r[0]+x0,r[1]+y0,r[2]+z0);
    }
    cerr << "\n Fehler! r0_case mit k<0 oder k>7." << endl;
    abort();
  }
  
  t_v3 E_func(const t_v3& v)
  {
    double tmp=norm(v);
    return t_v3(v/(tmp*tmp*tmp));
  }
  
  /*
    Elektrischer Feldstaerkevektor in einem Raumpunkt r:
  */
  t_v3 E(const t_v3& r)
  {

    t_v3 r0;
    t_v3 _E,tmp;
    _E=0.0;

    double x0,y0;
    int i,j,k;

    // (i,j), bzw. (x0,y0,+-c) : Punkte auf Plattenkondensator.
    for(i=0;i<n;i++) {
      x0=da*(0.5+i);
      for(j=0;j<m;j++) {
	y0=da*(0.5+j);
	tmp=0.0;
	for(k=0;k<4;k++) // Symmetrien des Plattenkondensators.
	  tmp += E_func(r0_case(r,x0,y0,c,k));
	for(k=-1;k>=-4;k--)
	  tmp -= E_func(r0_case(r,x0,y0,c,k));
	_E += tmp*C[j*n+i];
      }
    }
    return _E;
  }

  t_m3 E_Jac(const t_v3& v)
  {
    
  }

  /*////////////////////////////////////////////////////////////////////
    In der Funktion t_plattenC::init(...)
    werden Potentialkoeffizienten
  ////////////////////////////////////////////////////////////////////*/
  void  init(int _n, // Anzahl der Unterteilungen in x-Richtung
	     int _m, // Anzahl der Unterteilungen in y-Richtung
	     double _a, // halbe Abmessung in x-Richtung
	     double _c, // halber Plattenabstand
	     double _U0) // Potential einer Platte,
    // Spannungsdifferenz zwischen den Platten: 2U0
  {
    n=_n;
    m=_m;
    a=_a;
    c=_c;
    U0=_U0;
    c2=2.0*c;
    N=n*m; // Anzahl der Unbekannten des Gleichungssystems.
    da = a/n; // Seitenlaenge eines Randelements
    SIIconst = 2.0*sqrt(M_PI)/da; // Singulaeranteil des Randintegrals.

    // Lokale Variablen:
    int i,j,k,l,I,J;    
    double xi,yj,xk,yl;
    

    S = new (double*)[N] ; // Systemmatrix;
    for(I=0;I<N;I++) S[I]=new double[N+1];
    
    C = new double[N+1]; // Unbekannte (prop. Plattenladung)

    // Systemmatrix berechnen:
    for(i=0;i<n;i++)
      for(j=0;j<m;j++) {
	I=j*n+i;
	xi=da*(0.5+i); yj=da*(0.5+j);
	for(k=0;k<n;k++)
	  for(l=0;l<m;l++){
	    J=l*n+k;
	    xk=da*(0.5+k); yl=da*(0.5+l);
	    
	    if( I != J ) S[J][I]=green(xi-xk,yj-yl)-green(xi-xk,yj-yl,c2);
	    else S[J][I]=SIIconst;

	    S[J][I] += green(xi+xk,yj+yl) - green(xi+xk,yj+yl,c2)
	      + green(xi+xk,yj-yl) - green(xi+xk,yj-yl,c2)
	      + green(xi-xk,yj+yl) - green(xi-xk,yj+yl,c2);
	  }
	// Rechte Seite noch auffuellen:
	C[I]=U0;
      }
    
    // System loesen:
    
    if(!lin_solve_cholesky(N,S,C))
    {
      cerr << "\n Systemmatrix singulaer! Programmabbruch.";
      abort();
    }
    
    
  } // Ende von t_plattenC::init(...)
//////////////////////////////////////////////////////////////////////
  
  void write_Pot_Koeff(ostream& output)
  {
    int i,j;
    double xi,yj;
    for(i=0;i<n;i++)
      {
	xi=da*(0.5+i);
	for(j=0;j<m;j++)
	  {
	    yj=da*(0.5+j);
	    output << xi << ' ' << yj << ' ' << C[j*n+i] << '\n';
	  }
	output << '\n';
      }
  } // Ende von t_plattenC::write_Kap_Koeff()

}; // Ende von struct t_plattenC


/*
  Eine Hilfsfunktion zum Einlesen
  der geometrischen Daten des Plattenkondensators
*/
istream& operator >> (istream& is, const char* str) 
{
  string _str;
  do{
    is >> _str;
  } while( _str != str && is.good() );
  return is;
}

/*
  Einlesen der geometrischen Daten des Plattenkondensators
*/
istream& operator >> (ifstream& input,t_plattenC& plattenC)
{
  int n,m;
  double a,c,U0;
  input >> "n" >> n;
  input >> "m" >> m;
  input >> "a" >> a;
  input >> "c" >> c;
  input >> "U0" >> U0;
  plattenC.init(n,m,a,c,U0);
}

#ifdef MAIN
t_plattenC plattenC;

main(int nArg,char** arg)
{
  if(nArg!=2)
    {
      cerr << "\n Eingabedatei als Argument erwartet!"<< endl;
      exit(-1);
    }
  ifstream input(arg[1]); 
  if(!input)
    {
      cerr << "\n Eingabedatei konnte nicht geoeffnet werden!"<<endl;
      exit(-1);
    }
  
  input >> plattenC;
  cout
    << "\n Initialisierung abgeschlossen. (Potentialkoeff. ermittelt)"
    << endl;
  
  // Ladungskoeffizienten ausgeben:
  ofstream output("pot_koeff.dat");
  plattenC.write_Pot_Koeff(output);
  output.close();
  cout
    << "\n Potentialkoeffizienten in Datei pot_koeff.dat geschrieben."
    << endl;

  double xa,za,xb,zb,dx,dz;

  cout << "\n Berechnung des Potentials im Rechteck [xa,xb]x{0}x[za,zb]"
       << endl;

  if( !(input >> "xa" >> xa
	>> "xb" >> xb
	>> "dx" >> dx
	>> "za" >> za
	>> "zb" >> zb
	>> "dz" >> dz)
     )
    {
      cerr << "\n Fehler beim Einlesen des zu berechnenden Gebietes!" <<endl;
      abort();
    }

  cout << "\n Potentialverlauf wird in Datei pot.dat geschrieben."
       <<endl;
  
  double x,z;
  output.open("pot.dat");
  for(x=xa;x<=xb;x+=dx)
    {
      for(z=za;z<=zb;z+=dz)
	output << x << ' ' << z << ' ' << plattenC.pot(x,0.0,z) << '\n';
      output << '\n';
    }
  output.close();

  cout << "\n Feldstaerkeverlauf entlang x=xa...xb, y=0, z=0 wird in ez.dat geschrieben." << endl;
  output.open("ez.dat");
  for(x=xa;x<=xb;x+=dx)
    {
      output << x << ' ' << (plattenC.E(t_v3(x,0.0,0.0)))[2] << '\n';
    }
  output.close();

}

#endif
